/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;
using System.Text;
using System.Web;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.NodeDiscovery
{
    /// <summary>
    /// This implementation of <see cref="INodeDiscovery"/> will discover the current node from a
    /// <see cref="HttpRequest"/> for a given page. This  implementation will return a valid URL (with query string if
    /// specified) in <see cref="GetCurrentNode"/> method. The query string will be appended to the URL in a sorted
    /// order.
    /// </summary>
    /// <remarks>
    /// This class is thread safe in the context of the rendering process.
    /// </remarks>
    public class HttpRequestNodeDiscovery : INodeDiscovery
    {
        /// <summary>
        /// Creates an instance. Nothing done in this version, but that might change in an update.
        /// </summary>
        public HttpRequestNodeDiscovery()
        {
        }

        /// <summary>
        /// Discovers the node url from a <see cref="HttpRequest"/> on the page and creates a
        /// <see cref="BreadCrumbNode"/> from the resulting title, URL with query strings sorted.
        /// </summary>
        /// <param name="title">A possibly null, possibly empty title to use after trimming.</param>
        /// <param name="page">A non-null page to use.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="page"/> is null.</exception>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        /// <returns>A <c>BreadCrumbNode</c>.</returns>
        public BreadCrumbNode GetCurrentNode(string title, Page page)
        {
            if (page == null)
            {
                throw new ArgumentNullException("page");
            }

            try
            {
                HttpRequest request = page.Request;
                string url = request.Path;

                if (request.QueryString.Count != 0)
                {
                    StringBuilder builder = new StringBuilder();
                    // Sort the keys, then get them out one by one. This is more efficient than, for example, add
                    // each element to a sorted list.
                    string[] keys = request.QueryString.AllKeys;

                    Array.Sort(keys);

                    // The QueryString collection gives us decoded data
                    for (int i = 0; i < keys.Length; i++)
                    {
                        // Add a separator/initial character
                        builder.Append(i == 0 ? '?' : '&');
                        builder.Append(keys[i]);
                        builder.Append("=");
                        builder.Append(request.QueryString[keys[i]]);
                    }

                    url += builder.ToString();
                }

                return new BreadCrumbNode(title, url);
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error determining page node.", e);
            }
        }
    }
}
