/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;
using System.Text.RegularExpressions;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher
{
    /// <summary>
    /// This implementation of <c>INodeMatcher</c> will provide node matching services given a specific regex pattern.
    /// This class will return true when the passed node's url matches the specified pattern.
    /// </summary>
    /// <remarks>
    /// This class is thread safe by being immutable.
    /// </remarks>
    [Serializable]
    public class RegexNodeMatcher : AbstractNodeMatcher
    {
        /// <summary>
        /// Represents the pattern that will be used to determine if the node matches. This variable is set by the
        /// constructor, is immutable and can never be null or an empty string (after trimming). The pattern is exposed
        /// by the <see cref="NodePattern"/> property.
        /// </summary>
        private string nodePattern;

        /// <summary>
        /// Represents the prepared version of the nodePattern. This variable is set by the constructor, is immutable
        /// and can never be null. The pattern is used in <see cref="Matches"/>.
        /// </summary>
        private Regex pattern;

        /// <summary>
        /// Represents the pattern that will be used to determine if the node matches. This field is set by the
        /// constructor, is immutable and can never be null or an empty string (after trimming). The pattern is exposed
        /// by the <see cref="NodePattern"/> property.
        /// </summary>
        /// <value>
        /// A non-null, non-empty string, representing a regular expression.
        /// </value>
        public string NodePattern
        {
            get
            {
                return nodePattern;
            }
        }

        /// <summary>
        /// Constructs the <c>RegexNodeMatcher</c> from the given title, URL and pattern.
        /// </summary>
        /// <param name="title">A non-null, non-empty string (after trimming) to use for title.</param>
        /// <param name="url">A non-null, non-empty string (after trimming) to use for URL.</param>
        /// <param name="nodePattern">A non-null, non-empty string representing a valid regular expression.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="title"/>, <paramref name="url"/> or
        /// <paramref name="nodePattern"/> is null.
        /// </exception>
        /// <exception cref="ArgumentException">If <paramref name="title"/>, <paramref name="url"/> or
        /// <paramref name="nodePattern"/> is empty.
        /// </exception>
        /// <exception cref="ArgumentException">If the <paramref name="nodePattern"/> is not a valid regular
        /// expression.</exception>
        public RegexNodeMatcher(string title, string url, string nodePattern)
            : base(title, url)
        {
            if (nodePattern == null)
            {
                throw new ArgumentNullException("nodePattern");
            }
            if (nodePattern.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "nodePattern");
            }

            this.nodePattern = nodePattern;

            // We want to capture the complete string in the pattern. This way of doing it tells it to the engine
            // right off, which optimizes the actual matching. (In contrast to retrieving all matches and checking
            // whether there is any that covers the complete string.)
            pattern = new Regex("^" + nodePattern + "$", RegexOptions.Singleline);
        }

        /// <summary>
        /// Implements matching based on the specified pattern.
        /// </summary>
        /// <param name="node">The node to match.</param>
        /// <exception cref="ArgumentNullException">If the node URL is null.</exception>
        /// <returns>True if a match is present, false if not.</returns>
        public override bool Matches(BreadCrumbNode node)
        {
            if (node.Url == null)
            {
                throw new ArgumentNullException("node", "BreadCrumbNode not properly initialized.");
            }
            return pattern.IsMatch(node.Url);
        }

        /// <summary>
        /// Returns a hash code. Care has been taken to make permutations of the different strings not map to the same
        /// code.
        /// </summary>
        /// <returns>A hash code.</returns>
        public override int GetHashCode()
        {
            return Url.GetHashCode() ^ (NodePattern.GetHashCode() + 61) ^ (Title.GetHashCode() - 47);
        }

        /// <summary>
        /// Implements equality comparison for <see cref="RegexNodeMatcher"/> instances.
        /// </summary>
        /// <param name="obj">The object to compare against.</param>
        /// <returns>True if the objects are of the same type with the same values, false otherwise.</returns>
        public override bool Equals(object obj)
        {
            if (obj == null)
            {
                return false;
            }
            if (!obj.GetType().Equals(this.GetType()))
            {
                return false;
            }

            RegexNodeMatcher other = obj as RegexNodeMatcher;
            return other.NodePattern == NodePattern && other.Url == Url && other.Title == Title;
        }
    }
}
