/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{

    /// <summary>
    /// This class is a typical pair class that holds a paired association of a node matcher to the matching node. The
    /// node describes the actual node that matched the nodeMatcher and contains the contextual specific information.
    /// </summary>
    [Serializable]
    internal class NodePair
    {
        /// <summary>
        /// Represents the actual node that holds the contextual information (query string, etc) that matches the node
        /// matcher. This variable is set by the constructor, is immutable and can never be null. The node will be
        /// referenced in the associated property.
        /// </summary>
        private readonly BreadCrumbNode node;

        /// <summary>
        /// Represents the node matcher that holds the matching information and the default title. This variable is set
        /// by the constructor, is immutable and can be null (if there is no associated nodeMatcher). The path node
        /// will be referenced in the associated property and the <see cref="Node"/> property.
        /// </summary>
        private readonly INodeMatcher nodeMatcher;

        /// <summary>
        /// Represents the actual node that holds the contextual information (query string, etc) that matches the node
        /// matcher. This variable is set by the constructor, is immutable and can never be null.
        /// </summary>
        /// <remarks>
        /// A getter method for the node. This method will simply return the node if the node's title is non-null. If
        /// the node's title is null, a new node will be created with the title from the node matcher (if there is
        /// one).
        /// </remarks>
        /// <value>
        /// A <see cref="BreadCrumbNode"/>.
        /// </value>
        public BreadCrumbNode Node
        {
            get
            {
                // If we have no title, but we have a nodematcher, use it.
                if (node.Title == null && nodeMatcher != null)
                {
                    return new BreadCrumbNode(nodeMatcher.Title, node.Url);
                }
                else
                {
                    return node;
                }
            }
        }

        /// <summary>
        /// Represents the node matcher that holds the matching information and the default title. This value is set
        /// by the constructor, is immutable and can be null (if there is no associated nodeMatcher). In addition to
        /// this property, the value is also used by the <see cref="Node"/> property, if the title is null.
        /// </summary>
        /// <value>
        /// A <see cref="INodeMatcher"/>, or possibly null.
        /// </value>
        public INodeMatcher NodeMatcher
        {
            get
            {
                return nodeMatcher;
            }
        }

        /// <summary>
        /// Constructs the path pair from the given values.
        /// </summary>
        /// <param name="node">A non-null node.</param>
        /// <param name="nodeMatcher">A node matcher for the node, possibly null.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="node"/> has a null URL.</exception>
        internal NodePair(BreadCrumbNode node, INodeMatcher nodeMatcher)
        {
            if (node.Url == null)
            {
                throw new ArgumentNullException("node", "BreadCrumbNode not properly initialized.");
            }

            this.node = node;
            this.nodeMatcher = nodeMatcher;
        }
    }
}
