/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System; // For docs
using System.Collections;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// This class implements the "shortest path" retrieval logic for both <see cref="XmlPathDiscovery"/> and
    /// <see cref="DynamicPathDiscovery"/>.
    /// </summary>
    internal class PathDiscoveryHelper
    {
        /// <summary>
        /// For bookkeeping purposes, we sometimes need an empty node pair array. It makes sense to avoid allocating
        /// that array over and over again, when it's cheap to keep a single sentinel instance.
        /// </summary>
        private static readonly NodePair[] EMPTY_NODEPAIR_ARRAY = new NodePair[0];

        /// <summary>
        /// A simple helper that matches against a matcher, unless that matcher has already answered negative against
        /// the current node. A simple optimization that avoids iterating through the same nodes twice (or more...).
        /// </summary>
        /// <param name="currentNode">The current node.</param>
        /// <param name="matcher">The matcher to check against.</param>
        /// <param name="matchersChecked">The mapping of already checked matchers.</param>
        /// <returns>True or false.</returns>
        private static bool Matches(BreadCrumbNode currentNode, INodeMatcher matcher, IDictionary matchersChecked)
        {
            // A quick path, avoiding regex calls
            if (matchersChecked.Contains(matcher))
            {
                return (bool) matchersChecked[matcher];
            }

            bool matches = matcher.Matches(currentNode);
            matchersChecked[matcher] = matches;

            return matches;
        }

        /// <summary>
        /// This helper turns a sitemap, referencing parents to children, into a reversed sitemap, referencing children
        /// to parents.
        /// </summary>
        /// <param name="sitemap">The sitemap to reverse.</param>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        /// <returns>A reversed sitemap.</returns>
        public static IDictionary GetReversedMap(IDictionary sitemap)
        {
            IDictionary reversed = new Hashtable();

            foreach (DictionaryEntry entry in sitemap)
            {
                foreach (INodeMatcher child in (entry.Value as IList))
                {
                    // Create a new list if it doesn't already exist
                    IList list = reversed[child] as IList;
                    if (list == null)
                    {
                        list = new ArrayList();
                        reversed[child] = list;
                    }

                    // Add the parent
                    list.Add(entry.Key);
                }
            }

            return reversed;
        }

        /// <summary>
        /// A helper performing a shortest path search, trying to keep as much of the current path as possible. When
        /// used from the XML path discovery mechanism, there is never any current path. This is just a special case
        /// of the more general dynamic algorithm.
        /// </summary>
        /// <remarks>
        /// The search itself is performed by a BFS method. As all "distances" in the graph/tree are unitary, no
        /// complex priority queue is needed.
        /// </remarks>
        /// <param name="currentNode">The current node we are trying to find a path for.</param>
        /// <param name="oldPath">The old path to follow.</param>
        /// <param name="sitemap">The sitemap.</param>
        /// <param name="reversedSitemap">The reversed sitemap, highly useful for a shortest path search towards the
        /// root.</param>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        /// <exception cref="Exception">This is an internal helper, so some exceptions of other types may be thrown.
        /// Proper wrapping expected by the caller.</exception>
        /// <returns>A breadcrumb path, or null if the path would be empty.</returns>
        public static NodePair[] GetPath(BreadCrumbNode currentNode, NodePair[] oldPath,
            IDictionary sitemap, IDictionary reversedSitemap)
        {
            // Empty/null oldPath should be treated identically
            if (oldPath == null)
            {
                oldPath = EMPTY_NODEPAIR_ARRAY;
            }

            // This is the tentative return value
            NodePair[] newPath = null;

            // These node matchers have been checked and don't match the current node. This means that the shortest
            // path check is accelerated, by never checking the children of nodes in the current path, as they are
            // specifically checked before the search is initiated.
            IDictionary matchersChecked = new Hashtable();

            // If there is an old path, try to match the current node against it
            if (oldPath.Length != 0)
            {
                newPath = TrimPath(currentNode, oldPath, matchersChecked);

                // Still no path? See if we can extend it by following any part of the current path a single step down.
                if (newPath == null)
                {
                    newPath = ExtendPath(currentNode, oldPath, sitemap, matchersChecked);
                }
            }

            // Ok, now we get the big guns out, a real shortest path search.
            if (newPath == null)
            {
                // We keep a queue for the next and the current step. That way, we can clear everything already done.
                // The actual queue entries are the number of matched oldPath elements and a NodePair
                ArrayList[] queue = new ArrayList[2];
                queue[0] = new ArrayList();
                queue[1] = new ArrayList();

                // A dictionary to maintain what node matchers have already been visited, and from what node they were
                // entered (back-tracking)
                IDictionary back = new Hashtable();

                // A dictionary to maintain what node matchers have been visited in the current step, and their
                // position in the queue (needed for maximizing oldPath likeness)
                IDictionary inThis = new Hashtable();

                // oldPath expressed as number mappings instead.
                IDictionary pathMappings = new Hashtable();

                PrepareShortestPath(currentNode, oldPath, sitemap, matchersChecked, back, inThis, pathMappings,
                    queue[0]);
                INodeMatcher root = ShortestPath(queue, reversedSitemap, back, inThis, pathMappings);

                newPath = CreatePath(root, back, oldPath);
            }

            // Ok, no path to report.
            if (newPath == null || newPath.Length == 0)
            {
                return null;
            }

            // Try to make the last node identical to the current node, influences possibly the specific URL
            if (!newPath[newPath.Length - 1].Node.Equals(currentNode))
            {
                newPath[newPath.Length - 1] = new NodePair(currentNode, newPath[newPath.Length - 1].NodeMatcher);
            }

            return newPath;
        }

        /// <summary>
        /// A helper that uses the "backtracking" structure from the shortest path search to reconstruct the path,
        /// including the requirements to keep as much as possible of the old path, if there is one.
        /// </summary>
        /// <param name="root">The root we found in the search.</param>
        /// <param name="back">The back-tracking map.</param>
        /// <param name="oldPath">The old path.</param>
        /// <returns>A new path of <c>NodePair</c> instances.</returns>
        private static NodePair[] CreatePath(INodeMatcher root, IDictionary back, NodePair[] oldPath)
        {
            ArrayList path = new ArrayList();
            INodeMatcher node = root;
            while (node != null)
            {
                path.Add(new NodePair(new BreadCrumbNode(node.Title, node.Url), node));
                node = (INodeMatcher) back[node];
            }

            NodePair[] newPath = new NodePair[path.Count];
            path.CopyTo(newPath);

            for (int i = 0; i < newPath.Length; i++)
            {
                // Doesn't match, or already past old list end?
                if (oldPath.Length <= i || !newPath[i].NodeMatcher.Equals(oldPath[i].NodeMatcher))
                {
                    break;
                }

                // Transfer specific old path entries
                newPath[i] = oldPath[i];
            }

            return newPath;
        }

        /// <summary>
        /// The actual machinery of finding the shortest path.
        /// </summary>
        /// <param name="queue">The two queue lists we are working against.</param>
        /// <param name="reversedSitemap">The reversed site map.</param>
        /// <param name="back">The backtracking map, it's filled during our search.</param>
        /// <param name="inThis">The map coordinating what's already been visited/added in this step, at what cost.
        /// It's passed in here to reuse the structure, as it's always needed by <see cref="AddQueue"/>.</param>
        /// <param name="pathMappings">The mappings between path elements and positions in the old path, used to
        /// optimize the old path likeness.</param>
        /// <returns>The root of the shortest path. Can be used to backtrack through the results. Null if no root was
        /// found.</returns>
        private static INodeMatcher ShortestPath(IList[] queue, IDictionary reversedSitemap, IDictionary back,
            IDictionary inThis, IDictionary pathMappings)
        {
            int time = 0;
            int maxScore = -2;
            INodeMatcher root = null;

            while (root == null)
            {
                IList nowQueue = queue[time % 2];
                IList nextQueue = queue[(time + 1) % 2];

                // No root found?
                if (nowQueue.Count == 0)
                {
                    return null;
                }

                inThis.Clear();
                nextQueue.Clear();

                // Iterate through the queue, every second entry is a real parent to examine, the other entries are
                // old path match score values
                for (int i = 0; i < nowQueue.Count; i += 2)
                {
                    int oldPathPosition = (int) nowQueue[i];
                    INodeMatcher matcher = (INodeMatcher) nowQueue[i + 1];

                    IList parentList = reversedSitemap[matcher] as IList;
                    if (parentList == null)
                    {
                        if (oldPathPosition > maxScore)
                        {
                            maxScore = oldPathPosition;
                            root = matcher;
                        }
                    }
                    else
                    // Add parents
                    {
                        foreach (INodeMatcher parent in parentList)
                        {
                            AddQueue(parent, nextQueue, back, inThis, pathMappings, matcher, oldPathPosition);
                        }
                    }
                }
                time++;
            }
            return root;
        }

        /// <summary>
        /// Initializes the framework for finding the shortest path.
        /// </summary>
        /// <param name="currentNode">The current node, starting point of the search.</param>
        /// <param name="oldPath">The old path.</param>
        /// <param name="sitemap">The sitemap.</param>
        /// <param name="matchersChecked">The data about already checked <see cref="INodeMatcher"/> instances.</param>
        /// <param name="back">The backtracking map.</param>
        /// <param name="inThis">The data about already added nodes in this step, used in <see cref="AddQueue"/>.
        /// </param>
        /// <param name="pathMappings">The mappings between path elements and positions in the old path, used to
        /// optimize the old path likeness. Initialized in this method.</param>
        /// <param name="nowQueue">The first of the two queue lists.</param>
        private static void PrepareShortestPath(BreadCrumbNode currentNode, NodePair[] oldPath,
            IDictionary sitemap, IDictionary matchersChecked, IDictionary back, IDictionary inThis,
            IDictionary pathMappings, IList nowQueue)
        {
            for (int i = 0; i < oldPath.Length; i++)
            {
                if (oldPath[i].NodeMatcher != null)
                {
                    pathMappings[oldPath[i].NodeMatcher] = i;
                }
            }

            foreach (DictionaryEntry entry in sitemap)
            {
                // If we match this node, add it as a start point for the search.
                if (Matches(currentNode, (INodeMatcher) entry.Key, matchersChecked))
                {
                    AddQueue((INodeMatcher) entry.Key, nowQueue, back, inThis, pathMappings, null, -1);
                }
            }
        }

        /// <summary>
        /// Walking up along the old path, until we find a node with some children in the static map. Check if the
        /// current node matches any of those.
        /// </summary>
        /// <param name="currentNode">The current node.</param>
        /// <param name="oldPath">The old path.</param>
        /// <param name="sitemap">The sitemap, containing all known children for all nodes.</param>
        /// <param name="matchersChecked">The map of already checked matchers, to speed things up a little.</param>
        /// <returns>A new path, if it's found in this manner.</returns>
        private static NodePair[] ExtendPath(BreadCrumbNode currentNode, NodePair[] oldPath, IDictionary sitemap,
            IDictionary matchersChecked)
        {
            // Default.
            NodePair[] newPath = null;

            for (int i = oldPath.Length - 1; i >= 0; i--)
            {
                if (oldPath[i].NodeMatcher != null)
                {
                    IList children = sitemap[oldPath[i].NodeMatcher] as IList;
                    if (children == null)
                    {
                        continue;
                    }

                    // We have a list of children here.
                    foreach (INodeMatcher child in children)
                    {
                        if (Matches(currentNode, child, matchersChecked))
                        {
                            // Copy the old path list, add a new element for this "offspring"
                            newPath = new NodePair[i + 2];
                            for (int j = 0; j <= i; j++)
                            {
                                newPath[j] = oldPath[j];
                            }

                            // Add the node, and some children
                            newPath[i + 1] = new NodePair(currentNode, child);

                            // If we can do this extension at an earlier point, then that's a shorter path, so let's
                            // do so.
                        }
                    }
                }

                // We don't go farther up in the tree, because the risk that the path that we find isn't really
                // the shortest one increases significantly
                // If this break was removed, performance would improve slightly, and the user would stay on the same
                // path more frequently, even if it's not really the shortest one available. This could be made
                // configurable.
                break;
            }

            return newPath;
        }

        /// <summary>
        /// It's possible that the new path simply means walking up along the old path, to an "ancestor" to the
        /// previous node.
        /// </summary>
        /// <param name="currentNode">The current node.</param>
        /// <param name="oldPath">The old path.</param>
        /// <param name="matchersChecked">The map of already checked matchers, to speed things up a little.</param>
        /// <returns>A new path, if it's found in this manner.</returns>
        /// <returns>A new path, if it's found in this manner.</returns>
        private static NodePair[] TrimPath(BreadCrumbNode currentNode, NodePair[] oldPath, IDictionary matchersChecked)
        {
            // Default
            NodePair[] newPath = null;

            for (int i = oldPath.Length - 1; i >= 0; i--)
            {
                bool matching = false;

                // This could be written as a one-liner, but it could also quickly get confusing.
                // If we have a node matcher, that's used, if not, we match by URL directly.
                if (oldPath[i].NodeMatcher == null)
                {
                    matching = (oldPath[i].Node.Url == currentNode.Url);
                }
                else
                {
                    matching = Matches(currentNode, oldPath[i].NodeMatcher, matchersChecked);
                }

                if (matching)
                {
                    if (i == oldPath.Length - 1)
                    {
                        // No need to create a new list
                        newPath = oldPath;
                    }
                    else
                    {
                        newPath = new NodePair[i + 1];
                        for (int j = 0; j <= i; j++)
                        {
                            newPath[j] = oldPath[j];
                        }
                    }
                    // Now leave the loop, as we have our match.
                    break;
                }
            }

            return newPath;
        }

        /// <summary>
        /// "Simplify" a path from <see cref="NodePair"/> instances to <see cref="BreadCrumbNode"/>
        /// </summary>
        /// <param name="path">The <c>NodePair</c> array to convert.</param>
        /// <returns>An array of <c>BreadCrumbNode</c> instances.</returns>
        public static BreadCrumbNode[] SimplifyPath(NodePair[] path)
        {
            // Null goes to null, so does empty
            if (path == null || path.Length == 0)
            {
                return null;
            }

            BreadCrumbNode[] newPath = new BreadCrumbNode[path.Length];

            for (int i = 0; i < newPath.Length; i++)
            {
                newPath[i] = path[i].Node;
            }
            return newPath;
        }

        /// <summary>
        /// Adds an element to the queue, maximizing the old path likeness while we are at it.
        /// </summary>
        /// <param name="parent">The parent of the current node, that is, the node to add.</param>
        /// <param name="nextQueue">The queue to add it to.</param>
        /// <param name="back">The back-tracking map.</param>
        /// <param name="inThis">Data about node positions already present in this queue.</param>
        /// <param name="pathMappings">Path mappings against positions in the old path, higher is better.</param>
        /// <param name="matcher">The matcher we're working from, added in backtracking data.</param>
        /// <param name="oldPathPosition">The position of the current node, we don't tolerate gaps in the path.</param>
        private static void AddQueue(INodeMatcher parent, IList nextQueue, IDictionary back, IDictionary inThis,
            IDictionary pathMappings, INodeMatcher matcher, int oldPathPosition)
        {
            int score = -1;

            // In short: We've already visited this node in another path, with a shorter route, cut off the search
            // to optimize and avoid any possible loops
            if (back.Contains(parent) && !inThis.Contains(parent))
            {
                return;
            }
            if (pathMappings.Contains(parent))
            {
                score = (int) pathMappings[parent];
            }

            if (inThis.Contains(parent))
            {
                int position = (int) inThis[parent];

                // The existing entry is "better"
                if ((int) nextQueue[position] > score)
                {
                    return;
                }

                nextQueue[position] = score;
            }
            else
            {
                // Add a queue entry
                inThis[parent] = nextQueue.Count;
                nextQueue.Add(score);
                nextQueue.Add(parent);
            }

            // If the child score is a good score (part of the old trail), or there is no recorded back path, add it
            if (oldPathPosition == score + 1 || back[parent] == null)
            {
                back[parent] = matcher;
            }
        }
    }
}
