/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;
using System.Web;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// This path writer will write the node and title using the html anchor pattern of
    /// <c>&quot;&lt;a href='node.Url'&gt;node.Title&lt;/a&gt;</c>.
    /// </summary>
    /// <remarks>
    /// This path writer will be created by the Object Factory and utilized by the <see cref="BreadCrumbTrailControl"/>
    /// to write the current node/title using an html anchor. This class is thread safe in the context of the rendering
    /// process.
    /// </remarks>
    public class AnchorPathWriter : AbstractPathWriter
    {
        /// <summary>
        /// The JavaScript code used in event handlers to modify the CSS class of a text span. Note that the property
        /// is actually called className, not class, as class could be used as a keyword in the language itself.
        /// </summary>
        private const string CHANGE_CLASS_JSCRIPT = "this.className='{0}'";

        /// <summary>
        /// The attribute used for the "mouse over" event handler.
        /// </summary>
        private const string ONMOUSEOVER_ATTR = "onmouseover";

        /// <summary>
        /// The attribute used for the "mouse out" event handler.
        /// </summary>
        private const string ONMOUSEOUT_ATTR = "onmouseout";

        /// <summary>
        /// The title string used when no valid title was retrieved.
        /// </summary>
        private const string UNKNOWN_TITLE = "Unknown";

        /// <summary>
        /// Constructs the <see cref="AnchorPathWriter"/> using no constructor parameters.
        /// </summary>
        public AnchorPathWriter()
            : base()
        {
        }

        /// <summary>
        /// This method will write the specified node and style as a simple HTML reference (a href).
        /// </summary>
        /// <param name="writer">A non-null writer to use.</param>
        /// <param name="node">A non-null node to write.</param>
        /// <param name="nodeStyle">A style class to use for the node or null if no style should be used.</param>
        /// <param name="mouseOverStyle">The mouse over style class to use or null for none.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> or the URL of <paramref name="node"/>
        /// is null.
        /// </exception>
        /// <exception cref="BreadCrumbException">Any exception while writing.</exception>
        protected override void WriteNode(HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle,
            string mouseOverStyle)
        {
            if (node.Url == null)
            {
                throw new ArgumentNullException("node", "BreadCrumbNode not properly initialized.");
            }
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }

            try
            {
                // Our node discovery procedure will decode URLs, there is no good solution to this issue, as
                // information is lost in that process
                writer.AddAttribute(HtmlTextWriterAttribute.Href, node.Url);

                if (nodeStyle != null)
                {
                    writer.AddAttribute(HtmlTextWriterAttribute.Class, nodeStyle);
                }
                if (mouseOverStyle != null)
                {
                    writer.AddAttribute(ONMOUSEOVER_ATTR, string.Format(CHANGE_CLASS_JSCRIPT, mouseOverStyle));
                    string returnTo = (nodeStyle != null) ? nodeStyle : "";
                    writer.AddAttribute(ONMOUSEOUT_ATTR, string.Format(CHANGE_CLASS_JSCRIPT, returnTo));
                }

                writer.RenderBeginTag(HtmlTextWriterTag.A);

                string title = node.Title;
                if (title == null || title.Trim().Length == 0)
                {
                    title = UNKNOWN_TITLE;
                }

                writer.Write(HttpUtility.HtmlEncode(title));
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing HTML.", e);
            }
        }
    }
}
