/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 2.0
 */

using System;
using System.IO;
using System.Web.UI;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// This class tests one of the two provided path writers. Note that we test it sub-method by sub-method. Complete
    /// tests of the functionality provided by the abstract base class and the component itself isn't handled here.
    /// </summary>
    /// <remarks>
    /// We're inheriting from the tested class. It's stateless and we need access to the protected methods.
    /// </remarks>
    [TestFixture]
    public class TestAnchorPathWriter : AnchorPathWriter
    {
        /// <summary>
        /// Tests writing a single node, with no styles.
        /// </summary>
        [Test]
        public void TestWriteNode()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode("Title", "url"), null, null);
                }

                Assert.AreEqual("<a href=\"url\">Title</a>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing with a null writer, an exception is thrown-
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteNodeNullWriter()
        {
            WriteNode(null, new BreadCrumbNode("Title", "url"), null, null);
        }

        /// <summary>
        /// Tests writing with a "null" node, that is, a node with a null url. An exception should be thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteNodeNullNode()
        {
            using (StringWriter writer = new StringWriter())
            {
                try
                {
                    using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                    {
                        WriteNode(htmlWriter, new BreadCrumbNode(), null, null);
                    }
                }
                finally
                {
                    Assert.AreEqual("", writer.ToString(),
                        "Nothing should be wrtten.");
                }
            }
        }

        /// <summary>
        /// Tests writing a single node, with no styles and an empty title string. A predefined Unknown string is used.
        /// </summary>
        [Test]
        public void TestWriteEmptyTitle()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode(" ", "url"), null, null);
                }

                Assert.AreEqual("<a href=\"url\">Unknown</a>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with no styles and a null title string. A predefined Unknown string is used.
        /// </summary>
        [Test]
        public void TestWriteNullTitle()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode(null, "url"), null, null);
                }

                Assert.AreEqual("<a href=\"url\">Unknown</a>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with just a node style.
        /// </summary>
        [Test]
        public void TestWriteNodeStyle()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode("Title", "url"), "node", null);
                }

                Assert.AreEqual("<a href=\"url\" class=\"node\">Title</a>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with just a mouse over style.
        /// </summary>
        [Test]
        public void TestWriteMouseStyle()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode("Title", "url"), null, "mouse");
                }

                Assert.AreEqual(
                    "<a href=\"url\" onmouseover=\"this.className='mouse'\" onmouseout=\"this.className=''\">" +
                    "Title</a>",
                    writer.ToString(), "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with both style types.
        /// </summary>
        [Test]
        public void TestWriteBothStyles()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode("Title", "url"), "node", "mouse");
                }

                Assert.AreEqual(
                    "<a href=\"url\" class=\"node\" onmouseover=\"this.className='mouse'\" onmouseout=\"" +
                    "this.className='node'\">Title</a>", writer.ToString(), "HTML string not matching expectations.");
            }
        }
    }
}
